package UDPMulticast;
import java.io.*;
import java.net.*;
import java.util.*;
import UDPUnicast.*;
import Caching.*;
import StartAllServers.Debug;
import StartAllServers.*;
import Misc.Battery;
import HTTPMethods.*;

public class MulticastReceiveThread extends Thread {

    int multicast_socket_port;
    HashWrapper hWrap;
    String ipAddress;

    /**
     * Constructor which takes a port and an IP Address
     */
    public MulticastReceiveThread (int msp, String ipAddr) {
	super("MultiCastReceiveThread");
	this.multicast_socket_port = msp;
	ipAddress = ipAddr;
	hWrap = HashWrapper.getInstance();
    }
	
    /**
     * Method that runs the query receiver.  It does not receive
     * its own queries, it checks a hash table of the cache to see
     * if the file is there, and it processes aggregate queries.
     */
    public void run() {

	FileWrap fis = FileWrap.getInstance();

	//the socket to listen on
	try {
	    MulticastSocket socket = new MulticastSocket(multicast_socket_port);
if (Debug.DEBUG_UDPMulticast == true) { 
 	 System.out.println("Multicast Listening on: " + multicast_socket_port); 
 	}
	    //the group address
	    InetAddress address = InetAddress.getByName(StartAllServers.REPORT_REQUEST_GROUP_ADDRESS);
	    socket.joinGroup(address);
     
	    DatagramPacket packet;
	    
	    // continuously receive stuff
	    boolean test = true;
	    while(test) {
		  //end_time > System.currentTimeMillis()) {
		
		byte[] buf = new byte[1024];
		packet = new DatagramPacket(buf, buf.length);
		try {
		socket.receive(packet);

		String finTime = "Finish Query Transmission " + System.currentTimeMillis() + "\n";
		byte[] finT = new byte[finTime.length()];
		finTime.getBytes(0, finTime.length(), finT, 0);
		
		fis.write(finT);
		
		String startTime = "Start URL Extraction " + System.currentTimeMillis() + "\n";
		byte[] startT = new byte[startTime.length()];
		startTime.getBytes(0, startTime.length(), startT, 0);
		
		fis.write(startT);

		InetAddress origin_host =  packet.getAddress();
		if (Debug.DEBUG_UDPMulticast == true) { 
		    System.out.println("This packet originated from: " + origin_host.toString()); 
		}
		
		//don't accept packets you send yourself
		//		System.out.println("SOCKET: " + InetAddress.getByName(ipAddress));

		if (!InetAddress.getByName(ipAddress).equals(origin_host)) {
		    
		    
		    String received = new String(packet.getData());
		    if (Debug.DEBUG_UDPMulticast == true) { 
			System.out.println("********MULTICAST_RECEIVED: " + received); 
		    }
		    
		    //Here I have received a request 
		    //OUTLINE OF INTEGRATION REASONING WITH FANG's CODE

		    /*******************ADDED CODE*****************/
		    //Begin processing of aggregate query
		    StringTokenizer tok = new StringTokenizer(received, "\n");
		    while(tok.hasMoreTokens()){
			String fquery = tok.nextToken();
			int beg = fquery.indexOf("<Query>") + 7;
			int end = fquery.indexOf("</Query>");
			String query = fquery.substring(beg, end);
			
			Battery bt = Battery.getInstance();
			System.out.println("Before replying to multicast - ");
			bt.printBatteryLevel();

			String endTime = "End URL Extraction " + System.currentTimeMillis() + "\n";
			byte[] endT = new byte[endTime.length()];
			endTime.getBytes(0, endTime.length(), endT, 0);
			
			fis.write(endT);

			System.out.println("The query is: " + query);

			//check if query is in the hashtable
			String retval = null;
			boolean startswith = false;
			if(query.trim().startsWith("http://")){
			    String shashTime = "Start Hash Table " + System.currentTimeMillis() + "\n";
			    byte[] shashT = new byte[shashTime.length()];
			    shashTime.getBytes(0, shashTime.length(), shashT, 0);
			    
			    fis.write(shashT);
			    retval = (String)hWrap.get(new Integer(query.trim().hashCode()));

			    String ehashTime = "End Hash Table " + System.currentTimeMillis() + "\n";
			    byte[] ehashT = new byte[ehashTime.length()];
			    ehashTime.getBytes(0, ehashTime.length(), ehashT, 0);
			    
			    fis.write(ehashT);

			    startswith = true;
			    if(retval == null)
				System.out.println("Query is a url but is not in the cache");
			}else
			    System.out.println("Query is not a url");
			/**********************************************/
			if (bt.batteryOK()) {			
			    if (retval != null) {
				System.out.println("URL is in hashtable");
				SearchEngine sEngine = new SearchEngine();
				HandleWebPage wp = new HandleWebPage();
				String report;
				if(startswith && wp.FileisInCache(query.trim(), true)) {
				// report = sEngine.getDescriptor(ParseRequest.attachIndexHtml(query));

				    String repsTime = "Start Form Report " + query + " " + System.currentTimeMillis() + "\n";
				    byte[] repsT = new byte[repsTime.length()];
				    repsTime.getBytes(0, repsTime.length(), repsT, 0);
				    
				    fis.write(repsT);				    

				    report = sEngine.getURLFromCache(query);

				    String repxTime = "Finish Form Report " + query + " " + System.currentTimeMillis() + "\n";
				    byte[] repxT = new byte[repxTime.length()];
				    repxTime.getBytes(0, repxTime.length(), repxT, 0);
				    
				    fis.write(repxT);				    

				    String repTime = "Start Report Transmission " + query + " " + System.currentTimeMillis() + "\n";
				    byte[] repT = new byte[repTime.length()];
				    repTime.getBytes(0, repTime.length(), repT, 0);
				    
				    fis.write(repT);				    
				    
				    if (report != null) {
					UDPUnicastSender Sender = new UDPUnicastSender(origin_host.getHostAddress());
				//send it to a pre-determined port only once
					Sender.sendMess(report, 1);
				    }
				}
			    
				else if ((query != null) && ((report = sEngine.getReport(query)) != null)) {
				//report = sEngine.getReport(query);
				    
				    String repTime = "Start Report Transmission " + query + " " + System.currentTimeMillis() + "\n";
				    byte[] repT = new byte[repTime.length()];
				    repTime.getBytes(0, repTime.length(), repT, 0);
				    
				    fis.write(repT);				    
				    if (report != null) {
					UDPUnicastSender Sender = new UDPUnicastSender(origin_host.getHostAddress());
				//send it to a pre-determined port only once
					Sender.sendMess(report, 1);
				    }
				}
			    }else {}
			}
		    }
		}
		    //fulfill the request if it is in the Cache
		    //new ReplyerThread(received).start();
		    
		} catch (IOException e) {
		    if (Debug.DEBUG_UDPMulticast == true) { 
			System.out.println("MULTICAST_RECEIVE_THREAD - Receive Failed: " + e.getMessage() + "\nCurrent time is: " + System.currentTimeMillis()); 
		    }
		    continue;
		}
		
	    }
	    
	    
		socket.leaveGroup(address);
		socket.close();
	} catch (IOException e) {
	    if (Debug.DEBUG_UDPMulticast == true) { 
		System.out.println("MULTICAST_RECEIVE_THREAD - An IOException has been thrown: " + e.getMessage()); 
	    }
	}
    }
}








