package UDPMulticast;
import java.io.*;
import java.net.*;
import java.util.*;
import StartAllServers.Debug;
import StartAllServers.*;
import GUIServer.*;

public class MulticastSendThread extends Thread {

    DatagramSocket socket;      //socket to send packets
    String keywords;             //query kewords
    int num_times_to_send = -1;   //number of times to send query
    long until = -1;              //length of time to send query
    int datagram_port;             //port number on which to send query


    /**
     * Constructor that takes a port, query keywords, 
     * and the number of times to send the query.
     */
    public MulticastSendThread(int listen_port, String keyword, int ntts) {
	super("MulticastSendThread");
	int datagram_port = generate_port();
	this.keywords = keyword;
	this.num_times_to_send = ntts;
    }

    /**
     * Constructor that takes a port, query keywords,
     * and the amount of time to send the query.
     */
    public MulticastSendThread(int listen_port, String keyword, long until) {
	super("MulticastSendThread");
	int datagram_port = generate_port();
	this.keywords = keyword;
	this.until = until;
    }

    /** 
     * Method to randomly generate a port
     */
    synchronized public int generate_port() {
	int port = 0;
	while (port == 0)
	    try {
		port = 5000 + (int) (Math.random() * 1000); 
		socket = new DatagramSocket(port);
	    } catch (Exception e) {
if (Debug.DEBUG_UDPMulticast == true) { 
 	 System.out.println(e.getMessage()); 
 	}
		port = 0;
		continue;
	    }
	System.err.println("Using Datagram Port: " + port);
	return port;
    }

    /**
     * Method to send UDP requests.  Doesn't close sockets for 
     * efficiency and doesn't keep looking for new free ports.
     * Sends out aggregate queries, instead of single queries.
     */
    public void sendUDPRequest() {

	FileWrap fis = FileWrap.getInstance();

	try {
	    
	    byte[] buf = new byte[1024];
	    
	    String dString = null;
	    dString = "<Query>" + keywords + "</Query>";
	    
	    VectorWrapper vWrap = VectorWrapper.getInstance();
	    if(vWrap.indexOf(dString) == -1)
		vWrap.addElement(dString);    //store all current queries

	    /*******************ADDED CODE ***********************/
	    //create the new aggregate query
	    for(int i = 0; i<vWrap.size(); i++){
		if(i == 0)
		    dString = (String)vWrap.elementAt(i);
		else
		    dString = dString + "\n" + (String)vWrap.elementAt(i);
	    }
	    buf = dString.getBytes();
	    /*****************************************************/
	    if (Debug.DEBUG_UDPMulticast == true) { 
		System.out.println("MULTICAST_SEND_THREAD:" + keywords); 
	    }
	    
	    // send it
	    InetAddress group = InetAddress.getByName("230.0.0.1");
	    
	    //packet must originate from some other port
	    //since the multicaster is already using a port
	    //CHANGE THIS IF MULTICAST PORT HAS CHANGED
	    DatagramPacket packet = new DatagramPacket(buf, buf.length, group, StartAllServers.ASK_FOR_REPORTS_PORT);

	    String endTime = "Finish Form Query " + dString + " " + System.currentTimeMillis() + "\n";
	    byte[] endT = new byte[endTime.length()];
	    endTime.getBytes(0, endTime.length(), endT, 0);
	    
	    fis.write(endT);				    		

	    socket.send(packet);
	    
	    String repTime = "Start Query Transmission " + dString + " " + System.currentTimeMillis() + "\n";
	    byte[] repT = new byte[repTime.length()];
	    repTime.getBytes(0, repTime.length(), repT, 0);
	    
	    fis.write(repT);				    		
	    try {
		//sleep for 10 seconds
		Thread.sleep(10000);
	    } catch (InterruptedException e) {
		if (Debug.DEBUG_UDPMulticast == true) { 
		    System.out.println("Error with sleep: " + e.getMessage()); 
		}
	    }
	    
	} catch (IOException e) {
	    e.printStackTrace();
	  }
    }
    
    /**
     * Method to start query sending thread.  Continually sends
     * depending on number of times to send variable or length
     * of time variable.  Checks Query Report Buffer to ensure 
     * that the query has not been deleted
     */
    public void run() {
	
	int count = 0;
	//check which condition we're enforcing
	QueryReportBuffer mybuffer = QueryReportBuffer.getInstance();
	if (until == -1 && num_times_to_send > -1)
	    while (num_times_to_send > 0) {
		System.out.println("NUM TIMES: " + num_times_to_send);
		int loc = mybuffer.hasQuery(keywords);
		if(loc != -1){ 
		    sendUDPRequest();
		    num_times_to_send--;
		}else
		    break;
	    }
	
	else if (until > -1 && num_times_to_send == -1) {
	    while (System.currentTimeMillis() <  until){
		int loc = mybuffer.hasQuery(keywords);
		if(loc != -1){ 
		    sendUDPRequest();
		    num_times_to_send--;
		}else
		    break;
	    }
	}
	
	//else do nothing because something went wrong
	/*
	QueryReportBuffer mybuffer = QueryReportBuffer.getInstance();
	int loc = mybuffer.hasQuery(keywords);
	System.out.println("REMOVING QUERY");
	mybuffer.removeQuery(loc);
*/
	
          
	//close the socket we used to send UDP messages over
	socket.close();
	
    }
}

